'use strict'; // Import

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.wait = wait;
exports.isNode = isNode;
exports.isTTY = isTTY;
exports.bool = bool;
exports.useColors = useColors;
exports.color = color;
exports.inspect = inspect;
exports.inspectDiffResult = inspectDiffResult;
exports.diff = diff;
exports.compare = compare;
exports.inpectDiff = inpectDiff;
exports.log = log;
exports.logComparison = logComparison;
exports.equal = equal;
exports.deepEqual = deepEqual;
exports.contains = contains;
exports.errorEqual = errorEqual;
exports.returnViaCallback = returnViaCallback;
exports.completeViaCallback = completeViaCallback;
exports.errorViaCallback = errorViaCallback;
exports.returnErrorViaCallback = returnErrorViaCallback;
exports.throwErrorViaCallback = throwErrorViaCallback;
exports.expectViaCallback = expectViaCallback;
exports.expectErrorViaCallback = expectErrorViaCallback;
exports.expectThrowViaFunction = expectThrowViaFunction;
exports.expectFunctionToThrow = exports.expectErrorViaFunction = void 0;

var _util = _interopRequireDefault(require("util"));

var _assert = _interopRequireDefault(require("assert"));

var _ansicolors = _interopRequireDefault(require("ansicolors"));

var diffUtil = _interopRequireWildcard(require("diff"));

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var diffJson = diffUtil.diffJson || diffUtil.default.diffJson;
var diffChars = diffUtil.diffChars || diffUtil.default.diffChars;
/** Type for a callback that receives an optional error as the first argument */

/** Alias for setTimeout with paramaters reversed. */
function wait(delay, fn) {
  return setTimeout(fn, delay);
}
/** Whether or not we are running in the node environment */


function isNode() {
  return Boolean(typeof process !== 'undefined' && process.versions && process.versions.node);
}
/** Whether or not stdout and stderr are interactive. */


function isTTY() {
  return isNode() && process.stdout && process.stdout.isTTY === true && process.stderr && process.stderr.isTTY === true;
}
/** Convert a value to its boolean equivalent */


function bool(value) {
  if (value === 'no' || value === 'false' || value === 'n' || value === 'N') {
    return false;
  }

  if (value === 'null' || value === 'NULL') return null;
  return Boolean(value);
}
/** Whether or not colors are desired on this environment */


function useColors() {
  var env = isNode() && bool(process.env.COLOR || process.env.COLORS);
  if (typeof env === 'boolean') return env;
  if (isTTY()) return true;
  return isNode() && !process.argv.includes('--no-colors') && !process.argv.includes('--no-color');
}
/** Applies the color to the value if desired */


function color(value, color) {
  return useColors() ? color(value) : value;
}
/**
 * Return a stringified version of the value with indentation and colors where applicable.
 * Colors will be applied if the environment supports it (--no-colors not present and TTY).
 */


function inspect(value) {
  var opts = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  // If the terminal supports colours, and the user hasn't specified, then default to a sensible default
  var colors = useColors();
  var depth = 50; // Inspect and return using our defaults

  return _util.default.inspect(value, _objectSpread({
    colors: colors,
    depth: depth
  }, opts));
}
/** Return a highlighted string of a difference. */


function inspectDiffResult(diff) {
  var colors = useColors();
  var result = diff.reduce(function (accumulator, part) {
    var value = part.value;

    if (colors) {
      if (part.added) {
        value = _ansicolors.default.open.black + _ansicolors.default.bgGreen(value) + _ansicolors.default.open.green;
      } else if (part.removed) {
        value = _ansicolors.default.open.black + _ansicolors.default.bgBrightRed(value) + _ansicolors.default.open.green;
      }
    }

    return accumulator + value;
  }, '');
  return colors ? _ansicolors.default.green(result) : result;
}
/** Return the difference between the new data and the old data. */


function diff(newData, oldData) {
  if (typeof newData === 'object' && typeof oldData === 'object') {
    return diffJson(oldData, newData);
  } else {
    var a = inspect(oldData, {
      colors: false
    });
    var b = inspect(newData, {
      colors: false
    });
    return diffChars(a, b);
  }
}
/** Return the highlighted comparison between the new data and the old data. */


function compare(newData, oldData) {
  return inspectDiffResult(diff(newData, oldData));
}
/** Alias for {@link compare} */


function inpectDiff(newData, oldData) {
  return compare(newData, oldData);
}
/** Log the inspected values of each of the arguments to stdout */


function log() {
  if (isNode() && process.env.ASSERT_SILENCE) return;

  for (var i = 0; i < arguments.length; ++i) {
    console.log(inspect(i < 0 || arguments.length <= i ? undefined : arguments[i]));
  }
}
/** Output a comparison of the failed result to stderr */


function logComparison(actual, expected, error) {
  if (isNode() && process.env.ASSERT_SILENCE) return;
  var lines = ['------------------------------------', 'Comparison Error:', color(error.stack || error.message || error, _ansicolors.default.green), ''];
  lines.push('Comparison Diff:', compare(actual, expected), '');
  lines.push('Comparison Actual:', inspect(actual), '', 'Comparison Expected:', inspect(expected), '------------------------------------'); // Work for node

  if (isNode() && process.stderr) {
    process.stderr.write(lines.join('\n') + '\n');
  } // Work for browsers
  else {
      console.log(lines.join('\n'));
    }
}
/** Same as assert.equal in that it performs a strict equals check, but if a failure occurs it will output detailed information */


function equal(actual, expected) {
  var testName = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'equal assertion';
  var next = arguments.length > 3 ? arguments[3] : undefined;

  try {
    _assert.default.equal(actual, expected, testName);
  } catch (checkError) {
    logComparison(actual, expected, checkError);

    if (next) {
      next(checkError);
      return;
    } else {
      throw checkError;
    }
  }

  if (next) next();
}
/** Same as assert.deepEQual in that it performs a deep equals check, but if a failure occurs it will output detailed information */


function deepEqual(actual, expected) {
  var testName = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'deep equal assertion';
  var next = arguments.length > 3 ? arguments[3] : undefined;

  try {
    _assert.default.deepEqual(actual, expected, testName);
  } catch (checkError) {
    logComparison(actual, expected, checkError);

    if (next) {
      next(checkError);
      return;
    } else {
      throw checkError;
    }
  }

  if (next) next();
}
/** Checks to see if the actual result contains the expected result .*/


function contains(actual, expected) {
  var testName = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'contains assertion';
  var next = arguments.length > 3 ? arguments[3] : undefined;
  if (testName == null) testName = "Expected [".concat(actual, "] to contain [").concat(expected, "]");

  try {
    _assert.default.ok(actual.indexOf(expected) !== -1, testName);
  } catch (checkError) {
    if (next) {
      next(checkError);
      return;
    } else {
      throw checkError;
    }
  }

  if (next) next();
}
/** Checks to see if an error was as expected, if a failure occurs it will output detailed information */


function errorEqual(actualError, expectedError) {
  var testName = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'error equal assertion';
  var next = arguments.length > 3 ? arguments[3] : undefined;
  var expectedErrorMessage, actualErrorMessage;

  if (expectedError) {
    if (expectedError instanceof Error) {
      expectedErrorMessage = expectedError.message;
    } else {
      expectedErrorMessage = expectedError;
      expectedError = new Error(expectedErrorMessage);
    }
  }

  if (actualError) {
    if (actualError instanceof Error) {
      actualErrorMessage = actualError.message;
    } else {
      actualErrorMessage = actualError;
      actualError = new Error(actualErrorMessage);
    }
  }

  try {
    if (actualErrorMessage && expectedErrorMessage) {
      contains(actualErrorMessage, expectedErrorMessage, testName);
    } else {
      equal(actualError, expectedError || null, testName);
    }
  } catch (checkError) {
    logComparison(actualError && (actualError.stack || actualError.message || actualError), expectedErrorMessage, checkError);

    if (next) {
      next(checkError);
      return;
    } else {
      throw checkError;
    }
  }

  if (next) next();
}
/** Generate a callback that will return the specified value. */


function returnViaCallback(value) {
  return function () {
    return value;
  };
}
/** Generate a callback that will receive a completion callback whcih it will call with the specified result after the specified delay. */

/* eslint no-magic-numbers:0 */


function completeViaCallback(value) {
  var delay = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 100;
  return function (complete) {
    wait(delay, function () {
      complete(null, value);
    });
  };
}
/** Generate a callback that will receive a completion callback which it will call with the passed error after the specified delay. */

/* eslint no-magic-numbers:0 */


function errorViaCallback(error) {
  var delay = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 100;
  return function (complete) {
    wait(delay, function () {
      complete(error);
    });
  };
}
/** Generate a callback that return an error instance with the specified message/error. */


function returnErrorViaCallback() {
  var error = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'an error occured';
  return function () {
    if (error instanceof Error) {
      return error;
    } else {
      return new Error(error);
    }
  };
}
/** Generate a callback that throw an error instance with the specified message/error. */


function throwErrorViaCallback() {
  var error = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'an error occured';
  return function () {
    if (error instanceof Error) {
      throw error;
    } else {
      throw new Error(error);
    }
  };
}
/** Generate a callback that will check the arguments it received with the arguments specified, if a failure occurs it will output detailed information. */


function expectViaCallback() {
  for (var _len = arguments.length, expected = new Array(_len), _key = 0; _key < _len; _key++) {
    expected[_key] = arguments[_key];
  }

  return function () {
    for (var _len2 = arguments.length, actual = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
      actual[_key2] = arguments[_key2];
    }

    return deepEqual(actual, expected);
  };
}
/**
 * Generate a callback that will check its error (the actual error) against the passed error (the expected error).
 * If a failure occurs it will output detailed information. */


function expectErrorViaCallback(expected) {
  var testName = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'expect error via callback assertion';
  var next = arguments.length > 2 ? arguments[2] : undefined;
  return function (actual) {
    return errorEqual(actual, expected, testName, next);
  };
}
/** Expect the passed function to throw an error at some point. */


function expectThrowViaFunction(expected, fn) {
  var testName = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'expect error via function assertion';
  var next = arguments.length > 3 ? arguments[3] : undefined;
  var actual = null;

  try {
    fn();
  } catch (error) {
    actual = error;
  }

  errorEqual(actual, expected, testName, next);
}
/** Deprecated. Use {@link expectErrorViaFunction} instead. */


var expectErrorViaFunction = expectThrowViaFunction;
/** Deprecated. Use {@link expectErrorViaFunction} instead. */

exports.expectErrorViaFunction = expectErrorViaFunction;
var expectFunctionToThrow = expectThrowViaFunction;
exports.expectFunctionToThrow = expectFunctionToThrow;